@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for openSUSE 11.3's 2.6.34 kernel.
+# This is a kernel build script for openSUSE 11.2's 2.6.31 kernel.
 #
 
 die () {
@@ -60,24 +60,24 @@ fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-source-2.6.34.7-0.5.1.src.rpm ]
+if [ ! -r kernel-source-2.6.31.14-0.6.1.src.rpm ]
 then
-    wget http://download.opensuse.org/update/11.3/rpm/src/kernel-source-2.6.34.7-0.5.1.src.rpm || die "Can't download source package."
+    wget http://download.opensuse.org/update/11.2/rpm/src/kernel-source-2.6.31.14-0.6.1.src.rpm || die "Can't download source package."
 fi
-rpm --checksig kernel-source-2.6.34.7-0.5.1.src.rpm || die "Can't verify signature."
-rpm -ivh kernel-source-2.6.34.7-0.5.1.src.rpm || die "Can't install source package."
+rpm --checksig kernel-source-2.6.31.14-0.6.1.src.rpm || die "Can't verify signature."
+rpm -ivh kernel-source-2.6.31.14-0.6.1.src.rpm || die "Can't install source package."
 
-if [ ! -r kernel-default-2.6.34.7-0.5.1.nosrc.rpm ]
+if [ ! -r kernel-default-2.6.31.14-0.6.1.nosrc.rpm ]
 then
-    wget http://download.opensuse.org/update/11.3/rpm/src/kernel-default-2.6.34.7-0.5.1.nosrc.rpm || die "Can't download source package."
+    wget http://download.opensuse.org/update/11.2/rpm/src/kernel-default-2.6.31.14-0.6.1.nosrc.rpm || die "Can't download source package."
 fi
-rpm --checksig kernel-default-2.6.34.7-0.5.1.nosrc.rpm || die "Can't verify signature."
-rpm -ivh kernel-default-2.6.34.7-0.5.1.nosrc.rpm || die "Can't install source package."
+rpm --checksig kernel-default-2.6.31.14-0.6.1.nosrc.rpm || die "Can't verify signature."
+rpm -ivh kernel-default-2.6.31.14-0.6.1.nosrc.rpm || die "Can't install source package."
 
 cd /usr/src/packages/SOURCES/ || die "Can't chdir to /usr/src/packages/SOURCES/ ."
-if [ ! -r ccs-patch-1.8.0-20101231.tar.gz ]
+if [ ! -r ccs-patch-1.8.0-20110207.tar.gz ]
 then
-    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.0-20101231.tar.gz || die "Can't download patch."
+    wget -O ccs-patch-1.8.0-20110207.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.0-20110207.tar.gz' || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -92,24 +92,23 @@ patch << "EOF" || die "Can't patch spec file."
 -Name:           kernel-default
 +Name:           ccs-kernel-default
  Summary:        The Standard Kernel
- Version:        2.6.34.7
--Release:        0.5.1
-+Release:        0.5.1_tomoyo_1.8.0
+ Version:        2.6.31.14
+-Release:        0.6.1
++Release:        0.6.1_tomoyo_1.8.0p3
  %if %using_buildservice
  %else
  %endif
-@@ -306,6 +306,11 @@
- %endif
- 	%_sourcedir/series.conf .. $SYMBOLS
- 
+@@ -267,6 +267,10 @@
+     sed 's:^:patch -s -F0 -E -p1 --no-backup-if-mismatch -i ../:' \
+     >>../apply-patches.sh
+ bash -ex ../apply-patches.sh
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.8.0-20101231.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.34-suse-11.3.diff
++tar -zxf %_sourcedir/ccs-patch-1.8.0-20110207.tar.gz
++patch -sp1 < patches/ccs-patch-2.6.31-suse-11.2.diff
 +cat config.ccs >> ../config/%cpu_arch_flavor
-+
+ 
  cd %kernel_build_dir
  
- if [ -f %_sourcedir/localversion ] ; then
 EOF
 sed -e 's:^Provides:#Provides:' -e 's:^Obsoletes:#Obsoletes:' -e 's:-n kernel:-n ccs-kernel:' kernel-default.spec > ccs-kernel.spec || die "Can't edit spec file."
 echo ""
