@@ -1,13 +1,9 @@
 /*
  * security/ccsecurity/load_policy.c
  *
- * Copyright (C) 2005-2012  NTT DATA CORPORATION
- *
- * Version: 1.7.3+   2015/01/01
- *
- * This file is applicable to both 2.4.30 and 2.6.11 and later.
- * See README.ccs for ChangeLog.
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
+ * Version: 1.8.0+   2010/12/31
  */
 
 #include <linux/version.h>
@@ -15,17 +11,27 @@
 #include <linux/init.h>
 #include <linux/binfmts.h>
 #include <linux/sched.h>
-#include <linux/ccsecurity.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #include <linux/kmod.h>
+/*
+ * Regarding 2.4 kernels, we need to define __KERNEL_SYSCALLS__ in order to use
+ * waitpid() because call_usermodehelper() does not support UMH_WAIT_PROC.
+ */
 #define __KERNEL_SYSCALLS__
 #include <linux/unistd.h>
-#define ccs_lookup_flags  (LOOKUP_FOLLOW | LOOKUP_POSITIVE)
 #else
 #include <linux/fs.h>
 #include <linux/namei.h>
-#define ccs_lookup_flags LOOKUP_FOLLOW
 #endif
+#ifndef LOOKUP_POSITIVE
+#define LOOKUP_POSITIVE 0
+#endif
+
+/*
+ * TOMOYO specific part start.
+ */
+
+#include <linux/ccsecurity.h>
 
 /* Path to the policy loader. (default = CONFIG_CCSECURITY_DEFAULT_LOADER) */
 static const char *ccs_loader;
@@ -75,13 +81,13 @@ static _Bool ccs_policy_loader_exists(void)
 	 * exist. If the initrd includes /sbin/init but real-root-dev has not
 	 * mounted on / yet, activating MAC will block the system since
 	 * policies are not loaded yet.
-	 * Thus, let do_execve() call this function every time.
+	 * Thus, let do_execve() call this function everytime.
 	 */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 28)
 	struct path path;
 	if (!ccs_loader)
 		ccs_loader = CONFIG_CCSECURITY_DEFAULT_LOADER;
-	if (kern_path(ccs_loader, ccs_lookup_flags, &path) == 0) {
+	if (kern_path(ccs_loader, LOOKUP_FOLLOW | LOOKUP_POSITIVE, &path) == 0) {
 		path_put(&path);
 		return 1;
 	}
@@ -89,7 +95,8 @@ static _Bool ccs_policy_loader_exists(void)
 	struct nameidata nd;
 	if (!ccs_loader)
 		ccs_loader = CONFIG_CCSECURITY_DEFAULT_LOADER;
-	if (path_lookup(ccs_loader, ccs_lookup_flags, &nd) == 0) {
+	if (path_lookup(ccs_loader, LOOKUP_FOLLOW | LOOKUP_POSITIVE,
+			&nd) == 0) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 		path_put(&nd.path);
 #else
@@ -98,14 +105,15 @@ static _Bool ccs_policy_loader_exists(void)
 		return 1;
 	}
 #endif
-	printk(KERN_INFO "Not activating Mandatory Access Control now "
-	       "since %s doesn't exist.\n", ccs_loader);
+	printk(KERN_INFO "Not activating Mandatory Access Control "
+	       "as %s does not exist.\n", ccs_loader);
 	return 0;
 }
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+
 /**
- * ccs_run_loader - Start /sbin/ccs-init .
+ * ccs_run_loader - Start /sbin/ccs-init.
  *
  * @unused: Not used.
  *
@@ -124,6 +132,7 @@ static int ccs_run_loader(void *unused)
 	envp[2] = NULL;
 	return exec_usermodehelper(argv[0], argv, envp);
 }
+
 #endif
 
 /**
@@ -131,13 +140,13 @@ static int ccs_run_loader(void *unused)
  *
  * @filename: The program about to start.
  *
- * This function checks whether @filename is /sbin/init , and if so
+ * Returns nothing.
+ *
+ * This function checks whether @filename is /sbin/init, and if so
  * invoke /sbin/ccs-init and wait for the termination of /sbin/ccs-init
  * and then continues invocation of /sbin/init.
  * /sbin/ccs-init reads policy files in /etc/ccs/ directory and
  * writes to /proc/ccs/ interfaces.
- *
- * Returns nothing.
  */
 static void ccs_load_policy(const char *filename)
 {
@@ -159,11 +168,7 @@ static void ccs_load_policy(const char *filename)
 		envp[0] = "HOME=/";
 		envp[1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
 		envp[2] = NULL;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) || defined(UMH_WAIT_PROC)
-		call_usermodehelper(argv[0], argv, envp, UMH_WAIT_PROC);
-#else
 		call_usermodehelper(argv[0], argv, envp, 1);
-#endif
 	}
 #elif defined(TASK_DEAD)
 	{
@@ -224,7 +229,7 @@ static int __ccs_search_binary_handler(struct linux_binprm *bprm,
 	ccs_load_policy(bprm->filename);
 	/*
 	 * ccs_load_policy() executes /sbin/ccs-init if bprm->filename is
-	 * /sbin/init . /sbin/ccs-init executes /etc/ccs/ccs-load-module to
+	 * /sbin/init. /sbin/ccs-init executes /etc/ccs/ccs-load-module to
 	 * load loadable kernel module. The loadable kernel module modifies
 	 * "struct ccsecurity_ops". Thus, we need to transfer control to
 	 * __ccs_search_binary_handler() in security/ccsecurity/domain.c
@@ -240,37 +245,19 @@ static int __ccs_search_binary_handler(struct linux_binprm *bprm,
  * Some exports for loadable kernel module part.
  *
  * Although scripts/checkpatch.pl complains about use of "extern" in C file,
- * we don't put these into security/ccsecurity/compat.h because we want to
+ * we don't put these into security/ccsecurity/internal.h because we want to
  * split built-in part and loadable kernel module part.
  */
-extern asmlinkage long sys_getpid(void);
-extern asmlinkage long sys_getppid(void);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 35)
 extern spinlock_t vfsmount_lock;
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-static inline void module_put(struct module *module)
-{
-	if (module)
-		__MOD_DEC_USE_COUNT(module);
-}
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
-static void put_filesystem(struct file_system_type *fs)
-{
-	module_put(fs->owner);
-}
-#endif
-
-/* Jump table for loadable kernel module. */
+/* For exporting variables and functions. */
 const struct ccsecurity_exports ccsecurity_exports = {
 	.load_policy = ccs_load_policy,
-	.put_filesystem = put_filesystem,
-	.sys_getppid = sys_getppid,
-	.sys_getpid = sys_getpid,
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 35)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+	.__d_path = __d_path,
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 	.vfsmount_lock = &vfsmount_lock,
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
